//webkitURL is deprecated but nevertheless
URL = window.URL || window.webkitURL;

var gumStream; 						//stream from getUserMedia()
var recorder; 						//WebAudioRecorder object
var input; 							//MediaStreamAudioSourceNode  we'll be recording
var encodingType; 					//holds selected encoding for resulting audio (file)
var encodeAfterRecord = true;       // when to encode

// shim for AudioContext when it's not avb. 
var AudioContext = window.AudioContext || window.webkitAudioContext;
var audioContext; //new audio context to help us record

var encodingTypeSelect = document.getElementById("encodingTypeSelect");
var recordButton = document.getElementById("recordButton");
var stopButton = document.getElementById("stopButton");

audiorecord = document.getElementById("audio-record");

//add events to those 2 buttons
recordButton.addEventListener("click", startRecording);
stopButton.addEventListener("click", stopRecording);


// var audioPlayer = document.createElement('audio');

var title = document.getElementById("title");
var subject = document.getElementById("subject");
var contributor = document.getElementById("contributor");


// montrer le bouton enregister ou pas
function show_record(){
    if (title.value != '' && subject.value != '' && contributor.value != ''){
	      console.log('ok');
	      audiorecord.style.display = "block";
    }else{
	      audiorecord.style.display = "none";
    }
    
}

title.addEventListener("input", show_record);
subject.addEventListener("input", show_record);
contributor.addEventListener("input", show_record);
var enregistrez = document.getElementById("enregistrez");

function startRecording() {
    console.log("startRecording() called");

    /*
      Simple constraints object, for more advanced features see
      https://addpipe.com/blog/audio-constraints-getusermedia/
    */
    
    var constraints = { audio: true, video:false }

    /*
      We're using the standard promise based getUserMedia() 
      https://developer.mozilla.org/en-US/docs/Web/API/MediaDevices/getUserMedia
    */

    navigator.mediaDevices.getUserMedia(constraints).then(function(stream) {
	      //__log("getUserMedia() success, stream created, initializing WebAudioRecorder...");

	      /*
	        create an audio context after getUserMedia is called
	        sampleRate might change after getUserMedia is called, like it does on macOS when recording through AirPods
	        the sampleRate defaults to the one set in your OS for your playback device

	      */
	      audioContext = new AudioContext();
 
	      //update the format 
	      document.getElementById("formats").innerHTML="Format: 2 channel "+encodingTypeSelect.options[encodingTypeSelect.selectedIndex].value+" @ "+audioContext.sampleRate/1000+"kHz"

	      //assign to gumStream for later use
	      gumStream = stream;
	      
	      /* use the stream */
	      input = audioContext.createMediaStreamSource(stream);
	      
	      //stop the input from playing back through the speakers
	      //input.connect(audioContext.destination)

	      //get the encoding 
	      encodingType = encodingTypeSelect.options[encodingTypeSelect.selectedIndex].value;
	      
	      //disable the encoding selector
	      encodingTypeSelect.disabled = true;

	      recorder = new WebAudioRecorder(input, {
	          workerDir: "js/", // must end with slash
	          encoding: encodingType,
	          numChannels:2, //2 is the default, mp3 encoding supports only 2
	          onEncoderLoading: function(recorder, encoding) {
		            // show "loading encoder..." display
		            __log("Chargement de l'encodeur mp3 ...");
	          },
	          onEncoderLoaded: function(recorder, encoding) {
		            // hide "loading encoder..." display
		            __log("Encodeur mp3 chargé");
	          }
	      });

	      recorder.onComplete = function(recorder, blob) { 
	          __log("Encodage complet");
	          createDownloadLink(blob,recorder.encoding);
	          encodingTypeSelect.disabled = false;

            setTimeout(function(){
                Envoyer(5,audioPlayer.duration.toFixed(1));
            }, 500);
	      };

	      recorder.setOptions({
	          // timeLimit:120,
	          encodeAfterRecord:encodeAfterRecord,
	          ogg: {quality: 0.5},
	          mp3: {bitRate: 160}
	      });

	      //start the recording process
	      recorder.startRecording();

	      __log("L'enregistrement a commencé");

    }).catch(function(err) {
	      //enable the record button if getUSerMedia() fails
    	  recordButton.disabled = false;
    	  stopButton.disabled = true;

    });

    //disable the record button
    recordButton.disabled = true;
    stopButton.disabled = false;
}

function stopRecording() {
    console.log("stopRecording() called");
    //stop microphone access
    gumStream.getAudioTracks()[0].stop();
    stopButton.disabled = true;
    recordButton.disabled = false;
    //tell the recorder to finish the recording (stop recording + encode the recorded audio)
    recorder.finishRecording();
    // __log('Enregistrement arrêté');
    console.log("Enregistrement arrêté");
}

function createDownloadLink(blob,encoding) {
    
    var url = URL.createObjectURL(blob);
    var li = document.createElement('li');
    var link = document.createElement('a');

    // var filename = new Date().toISOString();
    var d = new Date();
    var y = d.getFullYear();
    var m = d.getMonth() + 1;
    var da = d.getDate();
    var h = d.getHours();
    var min = d.getMinutes();
    var filename = "Capsule-" + y + "-" + m + "-" + da + "-" + h + "h" + min + "min";

    //add controls to the <audio> element
    audioPlayer.controls = true;
    audioPlayer.src = url;

    audioPlayer.onended = function(){
        console.log("audio ended");
        Envoyer("6", "ended");
    };
    //link the a element to the blob
    link.href = url;
    link.download = filename + '.'+encoding;
    link.innerHTML = link.download;

    // add the new audio and a elements to the li element
    // li.appendChild(audioPlayer);
    //  audioPlayer();
    document.body.appendChild(audioPlayer);
    // li.appendChild(link);
    
    //upload link
    // var upload = document.createElement('a');
    // upload.href="#";
    // upload.innerHTML = "Enregistrer";

    enregistrez.style.display = "block"; // montrer le bouton
    enregistrez.addEventListener("click", function(event){
	      console.log("filename = " + filename);
	      var xhr=new XMLHttpRequest();
	      xhr.open("POST", window.location.origin+"/ajoutez", true);
	      
	      xhr.onload=function(e) {
	          if(this.readyState === 4) {
		            console.log("Server returned: ",e.target.responseText);
	          }
	      };
	      var fd = new FormData();
	      fd.append("audio_data", blob, filename);
	      fd.append("filename", filename);
	      fd.append("title", document.getElementById("title").value);
	      fd.append("subject", document.getElementById("subject").value);
	      fd.append("contributor", document.getElementById("contributor").value);
	      xhr.send(fd);
	      setTimeout(function(){
	          window.location.assign(window.location.origin + window.location.pathname);
	      }, 500);
    });
    //  li.appendChild(document.createTextNode ("       /       "))//add a space in between
    // add the li element to the ordered list
    // recordingsList.appendChild(li);
    // audiorecord.appendChild(upload)//add the upload link to li
}


//helper function
function __log(e, data) {
    log.innerHTML += "\n" + e + " " + (data || '');
}


function playRecording() {
    console.log("playRecording() called");
    audioPlayer.play();
}

function pauseRecording() {
    console.log("pauseRecording() called");
    audioPlayer.pause();
}

function saveRecording() {
    console.log("saveRecording() called... mais pas fait");
}

